@@ -23,13 +23,15 @@ class NetworkHelpers:
     emitters_and_consumers = {}
     with ThreadPoolExecutor() as executor:
       futures = [executor.submit(self.find_on_ip, ip_prefix + str(i)) for i in range(256)]
+      #futures = [executor.submit(self.find_on_ip, ip_address)]
       with tqdm(total=len(futures), desc='Scanning for Services', unit=' IP address') as pbar:
         for future in futures:
           devices = future.result()
           if devices is not None:
             for device in devices:
               emitters_and_consumers[device["id"]] = device
-            pbar.update(1)
+          pbar.update(1)
+    print("Scanning completed")
     return emitters_and_consumers
 
   def get_rabbitmq_local_ip(self):
@@ -48,6 +50,7 @@ class NetworkHelpers:
           pbar.update(1)
     if len(open_ips) == 0:
       return None
+    print("Scanning completed")
     return open_ips[0]
 
   def check_port(self, ip, port):
