@@ -16,9 +16,11 @@ class Beacon:
     self.db = TinyDB(self.project.id+'.json')
     self.queues = {}
     self.devices = {}
+    self.queueListeners = {}
     self.helper = NetworkHelpers()
     self.connection = None
     self.rabbitAutoconnect = False
+    self.rabbitConnecting = False
     print("[Beacon] Setting up the routes...")
     self.setupServerRoutes()
   
@@ -40,7 +42,7 @@ class Beacon:
       for deviceID in self.devices.keys():
         if deviceID != self.project.id:
           response = self.callService(self.devices[deviceID], "/ping")
-          if response is None:
+          if response is None or "id" not in response or response["id"] != deviceID:
             print("[Beacon] Device lost: ", self.devices[deviceID]["id"], self.devices[deviceID]["name"])
             rmList.append(deviceID)
       if len(rmList)>0:
@@ -68,7 +70,8 @@ class Beacon:
   # Auto-connect to Rabbit if offline or disconnected (every 5 sec)
   def rabbitAutoConnect(self):
     while True:
-      if self.rabbitAutoconnect:
+      if self.rabbitAutoconnect and self.rabbitConnecting is not True:
+        self.rabbitConnecting = True
         print("[Beacon] Scanning for RabbitMQ...")
         self.queueIP = self.helper.get_rabbitmq_local_ip()
         if self.queueIP is not None:
@@ -84,12 +87,15 @@ class Beacon:
       self.rabbitAutoconnect = True
     else:
       print("[Beacon] RabbitMQ found:", self.queueIP)
+      print("[Beacon] Connecting...")
       try:
         self.connection = pika.BlockingConnection(pika.ConnectionParameters(self.queueIP))
+        print("[Beacon] RabbitMQ connected:", self.queueIP)
+        self.rabbitConnecting = False
         # Upload the buffered data if any
         self.emitBufferedData()
       except Exception as e:
-        self.onRabbitClose()
+        self.onRabbitClose("connectToRabbit", e)
   
   # In case Rabbit closes
   def onRabbitClose(self):
@@ -154,13 +160,6 @@ class Beacon:
     self.addService(service)
     return {"ok": True}, 200
   
-  # Get a queue by name
-  def getQueue(self, name):
-    if name not in self.queues.keys():
-      self.queues[name] = self.connection.channel()
-      self.queues[name].queue_declare(queue=name, durable=True)
-    return self.queues[name]
-  
   # Buffer data in a local db in case the queue is offline
   def bufferData(self, name, data):
     self.db.insert({'queue': name, 'data': data, "sent": False})
@@ -206,11 +205,84 @@ class Beacon:
       channel.basic_publish(exchange='', routing_key=name, body=json.dumps(data))
       #print("[Beacon:sent]", name, data)
     except Exception as e:
-      self.onRabbitClose()
+      self.onRabbitClose("pushQueue", e)
       print("[Beacon:buffer]", name, data)
       #self.bufferData(name, data)
+  
+  # Get an array of queues used by mesh services
+  def getQueues(self):
+    queues = []
+    for deviceID in self.devices.keys():
+      if "queues" in self.devices[deviceID].keys():
+        for queue in self.devices[deviceID]["queues"]:
+          queues.append(queue)
+    return queues
 
+  # Get a queue by name
+  def getQueue(self, name):
+    try:
+      if name not in self.queues.keys():
+        self.queues[name] = self.connection.channel()
+        self.queues[name].queue_declare(queue=name, durable=True) #, passive=True
+      return self.queues[name]
+    except pika.exceptions.ConnectionWrongStateError as e:
+      print("[PIKA:ConnectionWrongStateError]", e)
+      return None
+    except Exception as e:
+      return None
+
+
+  # Subscribe to a queue in a new thread
+  # if name is "*", subscribe to all the queues
   def subscribe(self, name, callback):
-    channel = self.getQueue(name)
-    channel.basic_consume(queue=name, on_message_callback=callback, auto_ack=True)
-    channel.start_consuming()
\ No newline at end of file
+    print("[subscribe]", name)
+    if name == "*":
+      queues = self.getQueues()
+      print("[QUEUES]", queues)
+      for queue in queues:
+        self.subscribe(queue, callback)
+    else:
+      channel = self.getQueue(name)
+      if channel is not None:
+        self.pushQueue(name, {"ping": True})
+        channel.basic_consume(queue=name, on_message_callback=callback)
+        thread = threading.Thread(target=channel.start_consuming)
+        thread.daemon = True
+        thread.start()
+      else:
+        print("[Queue not found]", name)
+  
+  def onMessage(self, name, body):
+    print("[onMessage]", name, body)
+    if name in self.queueListeners:
+      for cb in self.queueListeners[name]:
+        print("[CB]", name, body)
+        cb(name, body)
+    if "*" in self.queueListeners:
+      for cb in self.queueListeners["*"]:
+        print("[CB]", name, body)
+        cb(name, body)
+
+  def listen(self, name):
+    print("[listen]", name)
+    if name == "*":
+      queues = self.getQueues()
+      print("[QUEUES]", queues)
+      for queue in queues:
+        self.listen(queue)
+    else:
+      try:
+        channel = self.getQueue(name)
+        if channel is not None:
+          fw = lambda ch, method, properties, body: self.onMessage(name, body)
+          channel.basic_consume(queue=name, on_message_callback=fw)
+          thread = threading.Thread(target=channel.start_consuming)
+          thread.daemon = True
+          thread.start()
+      except Exception as e:
+        print("[SUBFAIL]", name, e)
+
+  def onData(self, name, callback):
+    if name not in self.queueListeners:
+      self.queueListeners[name] = []
+    self.queueListeners[name].append(callback)
